package io.gitlab.jfronny.inceptum.cli.commands;

import io.gitlab.jfronny.commons.io.JFiles;
import io.gitlab.jfronny.inceptum.common.MetaHolder;
import io.gitlab.jfronny.inceptum.common.Utils;
import io.gitlab.jfronny.inceptum.cli.Command;
import io.gitlab.jfronny.inceptum.cli.CommandArgs;
import io.gitlab.jfronny.inceptum.launcher.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.launcher.util.InstanceLock;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;

public class ListCommand extends Command {
    public ListCommand() {
        super("Lists all available instances", "", "list", "ls");
    }

    @Override
    protected void invoke(CommandArgs args) throws IOException {
        List<Path> paths = JFiles.list(MetaHolder.INSTANCE_DIR);
        if (paths.isEmpty()) System.out.println("No instances are currently present");
        for (Path path : paths) {
            if (!Files.exists(path.resolve("instance.json"))) {
                System.out.println("- Invalid instance: " + path + " (no instance metadata)");
                continue;
            }
            System.out.println("- \"" + path.getFileName().toString() + "\"");
            if (InstanceLock.isSetupLocked(path)) {
                System.out.println("  Status: Setting up");
                continue;
            }
            InstanceMeta instance;
            try {
                instance = JFiles.readObject(path.resolve("instance.json"), InstanceMeta.class);
            } catch (IOException e) {
                Utils.LOGGER.error("  Could not load instance.json", e);
                continue;
            }
            System.out.println("  Status: " + (InstanceLock.isRunningLocked(path) ? "Running" : "Stopped"));
            System.out.println("  Version: " + instance.getMinecraftVersion());
            if (instance.isFabric()) System.out.println("  Fabric Loader: " + instance.getLoaderVersion());
            if (instance.java != null) System.out.println("  Custom Java: " + instance.java);
            if (instance.minMem != null || instance.maxMem != null)
                System.out.println("  Memory:" + (instance.minMem != null ? " Minimum: " + instance.minMem : "")
                        + (instance.maxMem != null ? " Maximum: " + instance.maxMem : ""));
        }
    }
}
