package io.gitlab.jfronny.inceptum.imgui.window.edit;

import imgui.ImGui;
import io.gitlab.jfronny.inceptum.common.R;
import io.gitlab.jfronny.inceptum.imgui.GuiMain;
import io.gitlab.jfronny.inceptum.imgui.control.Tab;
import io.gitlab.jfronny.inceptum.imgui.window.dialog.ProcessStateWatcherWindow;
import io.gitlab.jfronny.inceptum.imgui.window.dialog.TextBoxWindow;
import io.gitlab.jfronny.inceptum.launcher.LauncherEnv;
import io.gitlab.jfronny.inceptum.launcher.system.export.Exporter;
import io.gitlab.jfronny.inceptum.launcher.system.export.Exporters;
import io.gitlab.jfronny.inceptum.launcher.util.ProcessState;
import org.lwjgl.PointerBuffer;
import org.lwjgl.system.MemoryStack;
import org.lwjgl.util.tinyfd.TinyFileDialogs;

import java.nio.file.Path;
import java.nio.file.Paths;

public class ExportTab extends Tab {
    private final InstanceEditWindow window;

    public ExportTab(InstanceEditWindow window) {
        super("Export");
        this.window = window;
    }

    @Override
    protected void renderInner() {
        if (window.mds.isComplete()) {
            for (Exporter<?> exporter : Exporters.EXPORTERS) {
                if (ImGui.button(exporter.getName())) {
                    GuiMain.open(new TextBoxWindow("Version", "Please enter the current version of your modpack", "1.0", version -> {
                        String defaultName = window.name + " " + version + " (" + exporter.getName() + ")." + exporter.getFileExtension();
                        String filter = "*." + exporter.getFileExtension();
                        String file = saveFileDialog("Export " + exporter.getName() + " Pack", defaultName, filter, exporter.getName() + " packs (" + filter + ")");
                        if (file != null) {
                            ProcessState state = new ProcessState(Exporters.STEP_COUNT, "Initializing...");
                            GuiMain.open(new ProcessStateWatcherWindow("Exporting", "Could not export pack", state, cToken -> {
                                Path exportPath = Paths.get(file);
                                exporter.generate(state, window.path, window.instance, window.mds, exportPath, version);
                                LauncherEnv.showInfo(window.name + " has been successfully exported to " + exportPath, "Successfully exported");
                            }, null));
                        }
                    }, R::nop));
                }
            }
        } else {
            ImGui.text("The mods directory scan must be completed.\nThe progress for this can be observed in the mods tab");
        }
    }

    private String saveFileDialog(String title, String defaultName, String filter, String filterDescription) {
        try (MemoryStack stack = MemoryStack.stackPush()) {
            PointerBuffer filterPatterns = stack.mallocPointer(2);
            filterPatterns.put(stack.UTF8(filter));
            filterPatterns.flip();
            return TinyFileDialogs.tinyfd_saveFileDialog(title, defaultName, filterPatterns, filterDescription);
        }
    }
}
