package io.gitlab.jfronny.inceptum.launcher.system.export;

import io.gitlab.jfronny.commons.io.JFiles;
import io.gitlab.jfronny.inceptum.common.InceptumConfig;
import io.gitlab.jfronny.inceptum.launcher.model.curseforge.CurseforgeModpackManifest;
import io.gitlab.jfronny.inceptum.launcher.model.inceptum.InstanceMeta;
import io.gitlab.jfronny.inceptum.launcher.system.mds.IWModDescription;
import io.gitlab.jfronny.inceptum.launcher.system.source.CurseforgeModSource;
import io.gitlab.jfronny.inceptum.launcher.system.source.ModSource;
import io.gitlab.jfronny.inceptum.launcher.util.ModPath;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.LinkedHashSet;
import java.util.Set;

public class CurseForgeExporter extends Exporter<CurseforgeModpackManifest> {
    private static final String OVERRIDES_DIR_DEFAULT = "overrides";
    public CurseForgeExporter() {
        super("CurseForge", "zip", "overrides");
    }

    @Override
    protected CurseforgeModpackManifest generateManifests(Path root, InstanceMeta instance, String name, String version) throws IOException {
        CurseforgeModpackManifest manifest = new CurseforgeModpackManifest();
        manifest.minecraft = new CurseforgeModpackManifest.Minecraft();
        manifest.minecraft.version = instance.getMinecraftVersion();
        manifest.manifestType = "minecraftModpack";
        manifest.manifestVersion = 1;
        manifest.name = name;
        manifest.version = version;
        manifest.author = InceptumConfig.authorName;
        manifest.overrides = OVERRIDES_DIR_DEFAULT;
        manifest.minecraft.modLoaders = new LinkedHashSet<>();
        if (instance.isFabric()) {
            CurseforgeModpackManifest.Minecraft.ModLoader loader = new CurseforgeModpackManifest.Minecraft.ModLoader();
            loader.id = "fabric-" + instance.getLoaderVersion();
            loader.primary = true;
            manifest.minecraft.modLoaders.add(loader);
        }
        JFiles.writeObject(root.resolve("manifest.json"), manifest);
        return manifest;
    }

    @Override
    protected void addMods(Path root, InstanceMeta instance, Iterable<IWModDescription> mods, CurseforgeModpackManifest manifest, Path modsOverrides) throws IOException {
        modsLoop: for(IWModDescription mod : mods) {
            if (ModPath.isImod(mod.path())) {
                Set<ModSource> sources = mod.mod().orElseThrow().sources.keySet();
                for (ModSource source : sources) {
                    if (source instanceof CurseforgeModSource cms) {
                        manifest.files.add(cms.toManifest());
                        continue modsLoop;
                    }
                }
                // Not available on CF
                for (ModSource source : sources) {
                    Files.createDirectories(modsOverrides);
                    Files.copy(source.getJarPath(), modsOverrides.resolve(mod.path().getFileName().toString()));
                    continue modsLoop;
                }
            } else {
                Files.createDirectories(modsOverrides);
                Files.copy(mod.path(), modsOverrides.resolve(mod.path().getFileName().toString()));
                continue modsLoop;
            }
            throw new FileNotFoundException("Could not find mod file for " + mod.path());
        }
        JFiles.writeObject(root.resolve("manifest.json"), manifest);
    }
}
